/**
   This ast2infix() method takes an Abstract Syntax Tree
   for the language of arithmetic expressions and converts
   it into a string that contains the infix version of the
   expression.

   This version of ast2infix() uses knowledge about operator
   precedence (in infix notation) to use the minimun number
   of parentheses.
*/

public class AST2infix2
{
   /**
      This ast2infix() method is essentially
      a in-order traversal of the tree.
   */
   public static String ast2infix(Tree tree)
   {
      String result = "";

      if ( tree.degree() == 0 )  // constant term
      {
         result += tree.getElement();
      }
      else if ( tree.degree() == 1 )  // unary operator
      {
         if ( tree.getSubTree(0).degree() >= 2 )
         {
            result += tree.getElement() + "(" + ast2infix( tree.getSubTree(0) ) + ")";
         }
         else
         {
            result += tree.getElement() + ast2infix( tree.getSubTree(0) );
         }
      }
      else if ( tree.degree() == 2 )  // binary operator
      {
         int middlePrecedence = precedence( tree.getElement() );
         int leftPrecedence = precedence( tree.getSubTree(0).getElement() );

         if ( leftPrecedence > middlePrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(0) ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(0) );
         }

         result += " " + tree.getElement() + " ";

         int rightPrecedence = precedence( tree.getSubTree(1).getElement() );

         if ( rightPrecedence >= middlePrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(1) ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(1) );
         }
      }
      else  // operators with 3 or more operands
      {
         int operatorPrecedence = precedence( tree.getElement() );
         int leftPrecedence = precedence( tree.getSubTree(0).getElement() );

         if ( leftPrecedence > operatorPrecedence )
         {
            result += "(" + ast2infix( tree.getSubTree(0) ) + ")";
         }
         else
         {
            result += ast2infix( tree.getSubTree(0) );
         }

         for (int i = 1; i < tree.degree(); i++)
         {
            result += " " + tree.getElement() + " ";

            int rightPrecedence = precedence( tree.getSubTree(i).getElement() );

            if ( rightPrecedence >= operatorPrecedence )
            {
               result += "(" + ast2infix( tree.getSubTree(i) ) + ")";
            }
            else
            {
               result += ast2infix( tree.getSubTree(i) );
            }
         }
      }

      return result;
   }//ast2infix()


   // http://www.java-tips.org/java-se-tips/java.lang/what-is-java-operator-precedence.html
   // http://www.cs.princeton.edu/introcs/11precedence/
   private static int precedence(String op)
   {
      int result = 0;  // "highest" precedence (i.e., constants)

      if ( op.equals("^") )
      {
         result = 3;
      }
      else if ( op.equals("*")
             || op.equals("/")
             || op.equals("%") )
      {
         result = 4;
      }
      else if ( op.equals("+")
             || op.equals("-") )
      {
         result = 5;
      }

      return result;
   }//precedence()

}//AST2infix2