/*
  This program evaluates the following Tree, which
  represents the arithmetic expression 4*5 + 100/(15-5).

                 +
                / \
              /     \
             *       /
            / \    /   \
          4    5  100    -
                        / \
                      /     \
                     15      5
*/

public class Language_1_Example_1
{
   public static void main(String[] args)
   {
      // The string that represents the above expression tree.
      String exp = "( + ( * 4 5 ) ( / 100 ( - 15 5 ) ) )";

      // Print the expression.
      System.out.println( " Original expression = " + exp + "\n" );

      // Build the Tree data structure from its string representation.
      Tree tree = BuildTree.buildTree( exp );

      // Print the Tree as an S-expression
      System.out.println( tree + "\n" );

      // The "pretty printer" prints out the Tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the Tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( tree ) + "\n" );

      // The ast2infix() method prints out, in infix notation, the expression
      // represented by the expression tree . In a sense, the builTree()
      // and ast2infix() methods combine to define a language "translator".
      // They translates from the prefix expression language to the
      // infix expression language.
      System.out.println( "    Infix expression = " + AST2infix.ast2infix(  tree ) + "\n" );
      System.out.println( "    Infix expression = " + AST2infix2.ast2infix( tree ) + "\n" );

      // Print the tree traversals.
      System.out.println( " Pre-order traversal = " + Traverse.preOrder(  tree ) + "\n" );
      System.out.println( "Post-order traversal = " + Traverse.postOrder( tree ) + "\n" );


      // Interpret (evaluate) the Tree data structure.
      int result = Evaluate.eval( tree );
      System.out.println( tree + " ==> " + result + "\n" );


      // Create dot and png files from the btree.
      if(true)
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_1";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( Tree2dot_ver1.tree2dot(tree) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }

   }//main

}//Language_1_Example_1