/*
   This program parses, then evaluates, a number of examples
   of strings from the language of arithmetic expressions.
*/

public class Language_1_Example_3
{
   public static void main(String[] args)
   {
      String[] treeStrings = {
                              "( + 2 ( * 5 5 ) )",
                              "( + ( * 5 5 ) 2 )",
                              "( - ( * ( + 3 5 ) 4 ) 9 )",
                              "( - 9 ( * 4 ( + 5 3 ) ) )",
                              "( + ( / ( * 32 41 ) 2 ) ( * 12 48 ) )",
                              "( + 2 ( * 5 5 ) ( / 4 2 ) )",
                              "( + ( - 10 ) ( + 10 ) )",
                              "( * ( + 2 3 ) ( + 4 5 ) ( + 5 7 ) )",
                              "( + 1 ( ^ 2 1 ) ( ^ 2 2 ) ( ^ 2 3 ) ( ^ 2 4 ) ( ^ 2 5 ) ( ^ 2 6 ) )"
                             };

      int i = 0;
      for (i = 0; i < treeStrings.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         System.out.println( " Original expression = " + treeStrings[i] + "\n" );

         // Build the abstract syntax tree that represents the expression.
         Tree ast = BuildTree.buildTree( treeStrings[i] );

         // Print the AST as an S-expression
         System.out.println( ast + "\n" );

         // Pretty-print the abstract syntax tree (expression tree).
         System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

         // Print the infix version of the expression.
         System.out.println( "    Infix expression = " + AST2infix.ast2infix(  ast )  + "\n" );
         System.out.println( "    Infix expression = " + AST2infix2.ast2infix( ast )  + "\n" );

         // Print the tree traversals.
         System.out.println( " Pre-order traversal = " + Traverse.preOrder(  ast ) + "\n" );
         System.out.println( "Post-order traversal = " + Traverse.postOrder( ast ) + "\n" );

         // Evaluate the expression (interpret the AST).
         int result = Evaluate.eval( ast );
         System.out.println( ast + " ==> " + result + "\n" );


         // Create dot and png files from the ast.
         if(true)
         try
         {
            // Create the (empty) dot file.
            String baseName = String.format("Example_3.%02d", i);
            java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot"));
            // Put dot commands into the dot file
            out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
            out.close();

            // Create a command line for running dot.exe.
            String[] cmd = {"dot.exe",
                            "-Tpng",
                            baseName + ".dot",
                            "-o",
                            baseName + ".png"};
            // Execute the command line.
            java.lang.Runtime.getRuntime().exec(cmd);
         }
         catch (Exception e)
         {
            System.out.println( e );
         }
      }
   }//main

}//Language_1_Example_3