/**
   This program converts a BTree data structure into
   a DOT description of the tree. The DOT description
   can then be processed by the dot.exe command to
   produce a graphical image of the tree data struncture.

   Create a png file (from a dot file) with the following
   command line
   > dot.exe -Tpng -O tree.dot

   See
   http://www.graphviz.org/Documentation.php
*/

public class BTree2dot_ver1
{
   private static int nodeCount;

   public static String btree2dot(BTree bTree)
   {
      String result = "graph {\n";
      nodeCount = 0;
      result += btree2dot(bTree, nodeCount);
      result += "}\n";
      return result;
   }


   /**
      This btree2dot() method is essentially a
      pre-order traversal of the tree.
   */
   public static String btree2dot(BTree bTree, int nodeNumber)
   {
      String result = "";

      if ( 0 == bTree.degree() )  // this stops the recursion
      {
         result += "node" + nodeNumber + "[label=\"" + bTree.getElement() + "\"];\n";
      }
      else
      {
         // "process" the root node
         result += "node" + nodeNumber + "[label=\"" + bTree.getElement() + "\"];\n";

         // recursively traverse the left sub tree
         if ( null != bTree.getLeftTree() )
         {
            // create an edge
            result += "node" + nodeNumber + " -- " + "node" + (++nodeCount) + ";\n";
            // convert the sub tree into a dot description
            result += btree2dot(bTree.getLeftTree(),  nodeCount);
         }
         else  // there is no left branch
         {
            // create an invisible edge
            result += "node" + nodeNumber + " -- " + "node" + (++nodeCount);
            result += " [style=\"invisible\"];\n";
            // create an invisible, node
            result += "node" + nodeCount + " [style=\"invisible\"];\n";
         }

         // recursively traverse the right sub tree
         if ( null != bTree.getRightTree() )
         {
            // create an edge
            result += "node" + nodeNumber + " -- " + "node" + (++nodeCount) + ";\n";
            // convert the sub tree into a dot description
            result += btree2dot(bTree.getRightTree(), nodeCount);
         }
         else  // there is no right branch
         {
            // create an invisible edge
            result += "node" + nodeNumber + " -- " + "node" + (++nodeCount);
            result += " [style=\"invisible\"];\n";
            // create an invisible, node
            result += "node" + nodeCount + " [style=\"invisible\"];\n";
         }
      }
      return result;
   }

}//BTree2dot