/*
  This program evaluates the following BTree, which
  represents the arithmetic expression 4*5 + 100/(15-5).

                 +
                / \
              /     \
             *       /
            / \    /   \
          4    5  100    -
                        / \
                      /     \
                     15      5
*/

public class Language_1_Example_1
{
   public static void main(String[] args)
   {
      // The string that represents the above expression tree.
      String exp = "( + ( * 4 5 ) ( / 100 ( - 15 5 ) ) )";

      // Print the expression.
      System.out.println("Original expression = " + exp + "\n" );


      // Build the BTree datastructure from its string representation.
      BTree btree = BuildBTree.buildBTree( exp );


      // The "pretty printer" prints out the tree in a nice format.
      // In a sense, it reverses the process of parsing (and building)
      // the tree, so a "pretty-printer" is sometimes called an "unparser".
      System.out.println( PrettyPrinter.prettyPrint( btree ) + "\n" );


      // The ast2infix() method prints out, in infix notation, the expression
      // represented by the expression tree . In a sense, the builBTree()
      // and ast2infix() methods combine to define a language "translator".
      // They translates from the prefix expression language to the
      // infix expression language.
      System.out.println("Infix expression = " + AST2infix.ast2infix( btree ) + "\n" );
      System.out.println("Infix expression = " + AST2infix2.ast2infix( btree ) + "\n" );


      // Print the three tree traversals.
      System.out.println( "  In-order traversal = " + Traverse.inOrder(   btree ) + "\n" );
      System.out.println( " Pre-order traversal = " + Traverse.preOrder(  btree ) + "\n" );
      System.out.println( "Post-order traversal = " + Traverse.postOrder( btree ) + "\n" );


      // Interpret (evaluate) the BTree data structure.
      int result = Evaluate.eval( btree );
      System.out.println( exp + " ==> " + result + "\n" );


      // Create dot and png files from the btree.
      if(true)
      try
      {
         // Create the (empty) dot file.
         String baseName = "Example_1";
         java.io.PrintWriter out = new java.io.PrintWriter(
                                      new java.io.File(baseName + ".dot"));
         // Put dot commands into the dot file
         out.println( BTree2dot_ver1.btree2dot(btree) + "\n" );
         out.close();

         // Create a command line for running dot.exe.
         String[] cmd = {"dot.exe",
                         "-Tpng",
                         baseName + ".dot",
                         "-o",
                         baseName + ".png"};
         // Execute the command line.
         java.lang.Runtime.getRuntime().exec(cmd);
      }
      catch (Exception e)
      {
         System.out.println( e );
      }

   }//main

}//Language_1_Example_1